"""Pytest fixtures for mcp-roblox-docs tests."""

import pytest
from pathlib import Path
import tempfile
import shutil


@pytest.fixture
def temp_cache_dir():
    """Create a temporary cache directory for testing."""
    temp_dir = tempfile.mkdtemp(prefix="mcp-roblox-docs-test-")
    yield Path(temp_dir)
    # Cleanup after test
    shutil.rmtree(temp_dir, ignore_errors=True)


@pytest.fixture
def sample_class_data():
    """Sample class data for testing."""
    return [
        {
            "Name": "Part",
            "Superclass": "BasePart",
            "Members": [
                {
                    "MemberType": "Property",
                    "Name": "Anchored",
                    "ValueType": {"Name": "bool"},
                    "Tags": [],
                },
                {
                    "MemberType": "Method",
                    "Name": "Resize",
                    "Parameters": [
                        {"Name": "normalId", "Type": {"Name": "NormalId"}},
                        {"Name": "deltaAmount", "Type": {"Name": "int"}},
                    ],
                    "ReturnType": {"Name": "bool"},
                    "Tags": [],
                },
                {
                    "MemberType": "Event",
                    "Name": "Touched",
                    "Parameters": [{"Name": "otherPart", "Type": {"Name": "BasePart"}}],
                    "Tags": [],
                },
            ],
            "Tags": [],
        },
        {
            "Name": "TweenService",
            "Superclass": "Instance",
            "Members": [
                {
                    "MemberType": "Method",
                    "Name": "Create",
                    "Parameters": [
                        {"Name": "instance", "Type": {"Name": "Instance"}},
                        {"Name": "tweenInfo", "Type": {"Name": "TweenInfo"}},
                        {"Name": "propertyTable", "Type": {"Name": "Dictionary"}},
                    ],
                    "ReturnType": {"Name": "Tween"},
                    "Tags": [],
                },
            ],
            "Tags": ["Service", "NotCreatable"],
        },
    ]


@pytest.fixture
def sample_enum_data():
    """Sample enum data for testing."""
    return [
        {
            "Name": "Material",
            "Items": [
                {"Name": "Plastic", "Value": 256},
                {"Name": "Wood", "Value": 512},
                {"Name": "Slate", "Value": 800},
            ],
        },
        {
            "Name": "EasingStyle",
            "Items": [
                {"Name": "Linear", "Value": 0},
                {"Name": "Sine", "Value": 1},
                {"Name": "Quad", "Value": 2},
            ],
        },
    ]


@pytest.fixture
def sample_docs_data():
    """Sample documentation data for testing."""
    return {
        "Part": {"description": "A part is a fundamental building block."},
        "TweenService": {"description": "Service for creating tweens."},
    }



"""Tests for search index module."""

import pytest
from src.data.index import SearchIndex, SearchResult


class TestSearchIndex:
    """Tests for SearchIndex class."""

    def test_index_not_built_initially(self):
        """Test index starts not built."""
        idx = SearchIndex()
        assert not idx.is_built()

    def test_build_index(self, sample_class_data, sample_enum_data, sample_docs_data):
        """Test building the index."""
        idx = SearchIndex()
        idx.build(sample_class_data, sample_enum_data, sample_docs_data)

        assert idx.is_built()
        stats = idx.get_stats()
        assert stats["classes"] == 2
        assert stats["enums"] == 2

    def test_search_exact_match(self, sample_class_data, sample_enum_data, sample_docs_data):
        """Test exact search matches."""
        idx = SearchIndex()
        idx.build(sample_class_data, sample_enum_data, sample_docs_data)

        results = idx.search("Part", limit=5)
        assert len(results) > 0
        assert any(r.name == "Part" for r in results)

    def test_search_case_insensitive(self, sample_class_data, sample_enum_data, sample_docs_data):
        """Test search is case insensitive."""
        idx = SearchIndex()
        idx.build(sample_class_data, sample_enum_data, sample_docs_data)

        results = idx.search("part", limit=5)
        assert len(results) > 0
        assert any(r.name == "Part" for r in results)

    def test_search_member(self, sample_class_data, sample_enum_data, sample_docs_data):
        """Test searching for members."""
        idx = SearchIndex()
        idx.build(sample_class_data, sample_enum_data, sample_docs_data)

        results = idx.search("Anchored", limit=5)
        assert len(results) > 0
        # Should find the Anchored property
        member_results = [r for r in results if r.type == "member"]
        assert len(member_results) > 0

    def test_search_enum(self, sample_class_data, sample_enum_data, sample_docs_data):
        """Test searching for enums."""
        idx = SearchIndex()
        idx.build(sample_class_data, sample_enum_data, sample_docs_data)

        results = idx.search("Material", limit=5)
        assert len(results) > 0
        enum_results = [r for r in results if r.type == "enum"]
        assert len(enum_results) > 0

    def test_search_limit_respected(self, sample_class_data, sample_enum_data, sample_docs_data):
        """Test search limit is respected."""
        idx = SearchIndex()
        idx.build(sample_class_data, sample_enum_data, sample_docs_data)

        results = idx.search("a", limit=1)  # Generic query
        assert len(results) <= 1

    def test_search_no_results(self, sample_class_data, sample_enum_data, sample_docs_data):
        """Test search with no matches."""
        idx = SearchIndex()
        idx.build(sample_class_data, sample_enum_data, sample_docs_data)

        results = idx.search("xyznonexistent123")
        assert len(results) == 0

    def test_search_empty_query(self, sample_class_data, sample_enum_data, sample_docs_data):
        """Test search with empty query."""
        idx = SearchIndex()
        idx.build(sample_class_data, sample_enum_data, sample_docs_data)

        results = idx.search("")
        assert len(results) == 0


class TestFuzzySearch:
    """Tests for fuzzy search functionality."""

    def test_fuzzy_search_typo(self, sample_class_data, sample_enum_data, sample_docs_data):
        """Test fuzzy search finds results with typos."""
        idx = SearchIndex()
        idx.build(sample_class_data, sample_enum_data, sample_docs_data)

        # "Tweenservice" instead of "TweenService"
        results = idx.fuzzy_search("Tweenservice", limit=5)
        assert len(results) > 0
        assert any("Tween" in r.name for r in results)

    def test_fuzzy_search_partial(self, sample_class_data, sample_enum_data, sample_docs_data):
        """Test fuzzy search with partial match."""
        idx = SearchIndex()
        idx.build(sample_class_data, sample_enum_data, sample_docs_data)

        results = idx.fuzzy_search("Prt", limit=5, threshold=0.5)
        # Should potentially find "Part"
        # Note: With very short strings, fuzzy matching may not always work
        # This test verifies the function runs without error

    def test_fuzzy_match_threshold(self):
        """Test fuzzy match with different thresholds."""
        idx = SearchIndex()

        # Test the internal fuzzy match function
        score_high = idx._fuzzy_match("part", "part", threshold=0.9)
        assert score_high == 1.0

        score_similar = idx._fuzzy_match("part", "prt", threshold=0.5)
        # "part" vs "prt" should have decent similarity
        assert score_similar > 0 or score_similar == 0  # Depends on threshold

    def test_search_with_fuzzy_fallback_exact(
        self, sample_class_data, sample_enum_data, sample_docs_data
    ):
        """Test fuzzy fallback uses exact first."""
        idx = SearchIndex()
        idx.build(sample_class_data, sample_enum_data, sample_docs_data)

        results, used_fuzzy = idx.search_with_fuzzy_fallback("Part")
        assert len(results) > 0
        assert not used_fuzzy  # Should find exact match

    def test_search_with_fuzzy_fallback_fuzzy(
        self, sample_class_data, sample_enum_data, sample_docs_data
    ):
        """Test fuzzy fallback falls back to fuzzy."""
        idx = SearchIndex()
        idx.build(sample_class_data, sample_enum_data, sample_docs_data)

        # Search for something that won't match exactly
        results, used_fuzzy = idx.search_with_fuzzy_fallback("Tweenservce")
        # May or may not find results depending on threshold
        # But used_fuzzy should be True if no exact results
        if len(results) > 0 or used_fuzzy:
            # Either found fuzzy results or correctly indicated fuzzy was tried
            pass


class TestSearchResult:
    """Tests for SearchResult dataclass."""

    def test_search_result_creation(self):
        """Test creating a SearchResult."""
        result = SearchResult(
            type="class",
            name="Part",
            class_name=None,
            description="A part object",
            score=1.0,
            tags=["NotCreatable"],
        )

        assert result.type == "class"
        assert result.name == "Part"
        assert result.score == 1.0
        assert "NotCreatable" in result.tags

    def test_search_result_member(self):
        """Test SearchResult for a member."""
        result = SearchResult(
            type="member",
            name="Anchored",
            class_name="Part",
            description="Property: bool",
            score=0.5,
            member_type="Property",
        )

        assert result.type == "member"
        assert result.class_name == "Part"
        assert result.member_type == "Property"



"""Tests for input validation module."""

import pytest
from src.utils.validation import (
    validate_query,
    validate_class_name,
    validate_member_name,
    validate_enum_name,
    validate_topic_name,
    validate_limit,
    validate_operation_id,
    validate_flag_name,
    MAX_QUERY_LENGTH,
    MAX_NAME_LENGTH,
)


class TestValidateQuery:
    """Tests for validate_query function."""

    def test_valid_query(self):
        """Test valid query passes."""
        result = validate_query("TweenService")
        assert result == "TweenService"

    def test_query_with_spaces(self):
        """Test query with spaces."""
        result = validate_query("tween service animation")
        assert result == "tween service animation"

    def test_query_strips_whitespace(self):
        """Test whitespace is stripped."""
        result = validate_query("  hello  ")
        assert result == "hello"

    def test_empty_query_raises(self):
        """Test empty query raises ValueError."""
        with pytest.raises(ValueError, match="cannot be empty"):
            validate_query("")

    def test_whitespace_only_query_raises(self):
        """Test whitespace-only query raises ValueError."""
        with pytest.raises(ValueError, match="cannot be empty"):
            validate_query("   ")

    def test_none_query_raises(self):
        """Test None query raises ValueError."""
        with pytest.raises(ValueError, match="cannot be empty"):
            validate_query(None)

    def test_query_truncated_at_max_length(self):
        """Test long query is truncated."""
        long_query = "a" * (MAX_QUERY_LENGTH + 100)
        result = validate_query(long_query)
        assert len(result) == MAX_QUERY_LENGTH


class TestValidateClassName:
    """Tests for validate_class_name function."""

    def test_valid_class_name(self):
        """Test valid class name passes."""
        result = validate_class_name("TweenService")
        assert result == "TweenService"

    def test_class_name_with_underscore(self):
        """Test class name with underscore."""
        result = validate_class_name("Some_Class")
        assert result == "Some_Class"

    def test_class_name_with_numbers(self):
        """Test class name with numbers."""
        result = validate_class_name("Vector3")
        assert result == "Vector3"

    def test_empty_class_name_raises(self):
        """Test empty class name raises ValueError."""
        with pytest.raises(ValueError, match="cannot be empty"):
            validate_class_name("")

    def test_invalid_characters_raises(self):
        """Test invalid characters raise ValueError."""
        with pytest.raises(ValueError, match="Invalid class name"):
            validate_class_name("Part.Anchored")

    def test_spaces_raise(self):
        """Test spaces raise ValueError."""
        with pytest.raises(ValueError, match="Invalid class name"):
            validate_class_name("Tween Service")

    def test_truncated_at_max_length(self):
        """Test long name is truncated before validation."""
        long_name = "A" * (MAX_NAME_LENGTH + 10)
        result = validate_class_name(long_name)
        assert len(result) == MAX_NAME_LENGTH


class TestValidateMemberName:
    """Tests for validate_member_name function."""

    def test_valid_member_name(self):
        """Test valid member name passes."""
        result = validate_member_name("Anchored")
        assert result == "Anchored"

    def test_empty_raises(self):
        """Test empty member name raises ValueError."""
        with pytest.raises(ValueError, match="cannot be empty"):
            validate_member_name("")

    def test_invalid_characters_raises(self):
        """Test invalid characters raise ValueError."""
        with pytest.raises(ValueError, match="Invalid member name"):
            validate_member_name("Get:Value")


class TestValidateEnumName:
    """Tests for validate_enum_name function."""

    def test_valid_enum_name(self):
        """Test valid enum name passes."""
        result = validate_enum_name("Material")
        assert result == "Material"

    def test_empty_raises(self):
        """Test empty enum name raises ValueError."""
        with pytest.raises(ValueError, match="cannot be empty"):
            validate_enum_name("")


class TestValidateTopicName:
    """Tests for validate_topic_name function."""

    def test_valid_topic(self):
        """Test valid topic name passes."""
        result = validate_topic_name("tables")
        assert result == "tables"

    def test_topic_with_hyphens(self):
        """Test topic with hyphens passes."""
        result = validate_topic_name("type-checking")
        assert result == "type-checking"

    def test_topic_normalized_to_lowercase(self):
        """Test topic is normalized to lowercase."""
        result = validate_topic_name("Type-Checking")
        assert result == "type-checking"

    def test_topic_with_underscores_converted(self):
        """Test underscores work as identifiers."""
        result = validate_topic_name("control_structures")
        assert result == "control_structures"

    def test_empty_raises(self):
        """Test empty topic raises ValueError."""
        with pytest.raises(ValueError, match="cannot be empty"):
            validate_topic_name("")


class TestValidateLimit:
    """Tests for validate_limit function."""

    def test_valid_limit(self):
        """Test valid limit passes through."""
        result = validate_limit(25)
        assert result == 25

    def test_none_returns_default(self):
        """Test None returns default value."""
        result = validate_limit(None, default=25)
        assert result == 25

    def test_below_min_clamped(self):
        """Test value below min is clamped."""
        result = validate_limit(0, min_val=1, max_val=50)
        assert result == 1

    def test_above_max_clamped(self):
        """Test value above max is clamped."""
        result = validate_limit(100, min_val=1, max_val=50)
        assert result == 50

    def test_custom_range(self):
        """Test custom min/max range."""
        result = validate_limit(15, min_val=10, max_val=20)
        assert result == 15


class TestValidateOperationId:
    """Tests for validate_operation_id function."""

    def test_valid_operation_id(self):
        """Test valid operation ID passes."""
        result = validate_operation_id("DataStore_GetEntry")
        assert result == "DataStore_GetEntry"

    def test_empty_raises(self):
        """Test empty operation ID raises ValueError."""
        with pytest.raises(ValueError, match="cannot be empty"):
            validate_operation_id("")


class TestValidateFlagName:
    """Tests for validate_flag_name function."""

    def test_valid_flag_name(self):
        """Test valid flag name passes."""
        result = validate_flag_name("DFFlagDebugVisualize")
        assert result == "DFFlagDebugVisualize"

    def test_empty_raises(self):
        """Test empty flag name raises ValueError."""
        with pytest.raises(ValueError, match="cannot be empty"):
            validate_flag_name("")

    def test_invalid_characters_raises(self):
        """Test invalid characters raise ValueError."""
        with pytest.raises(ValueError, match="Invalid flag name"):
            validate_flag_name("FFlag-Invalid")



"""Tests for DevForum cache persistence."""

import pytest
import json
from datetime import datetime, timezone, timedelta
from pathlib import Path

from src.server import (
    _load_devforum_cache,
    _save_devforum_cache,
    _get_devforum_cache_path,
    _devforum_cache,
    _DEVFORUM_CACHE_TTL,
)


class TestDevForumCachePersistence:
    """Tests for DevForum cache save/load functionality."""

    def test_cache_file_path_exists(self):
        """Test cache file path is valid."""
        path = _get_devforum_cache_path()
        assert path.name == "devforum_cache.json"
        assert path.parent.exists() or True  # May not exist yet

    def test_save_and_load_cache(self, temp_cache_dir, monkeypatch):
        """Test saving and loading cache."""
        import src.server as server_module

        # Patch the cache directory
        test_cache_path = temp_cache_dir / "devforum_cache.json"
        monkeypatch.setattr(server_module, "_get_devforum_cache_path", lambda: test_cache_path)
        monkeypatch.setattr(server_module, "_devforum_cache_loaded", False)

        # Clear existing cache
        server_module._devforum_cache.clear()

        # Add test data
        test_time = datetime.now(timezone.utc)
        test_results = [{"id": 1, "title": "Test Topic"}]
        server_module._devforum_cache["test query"] = (test_time, test_results)

        # Save
        _save_devforum_cache()

        # Verify file exists
        assert test_cache_path.exists()

        # Read and verify content
        data = json.loads(test_cache_path.read_text())
        assert data["version"] == 1
        assert "test query" in data["entries"]

    def test_load_invalid_cache_file(self, temp_cache_dir, monkeypatch):
        """Test loading corrupted cache file doesn't crash."""
        import src.server as server_module

        test_cache_path = temp_cache_dir / "devforum_cache.json"
        monkeypatch.setattr(server_module, "_get_devforum_cache_path", lambda: test_cache_path)
        monkeypatch.setattr(server_module, "_devforum_cache_loaded", False)
        server_module._devforum_cache.clear()

        # Write invalid JSON
        test_cache_path.write_text("not valid json {{{")

        # Should not raise, just log warning
        _load_devforum_cache()

        # Cache should still work (empty)
        assert server_module._devforum_cache_loaded

    def test_load_expired_entries_ignored(self, temp_cache_dir, monkeypatch):
        """Test expired entries are not loaded."""
        import src.server as server_module

        test_cache_path = temp_cache_dir / "devforum_cache.json"
        monkeypatch.setattr(server_module, "_get_devforum_cache_path", lambda: test_cache_path)
        monkeypatch.setattr(server_module, "_devforum_cache_loaded", False)
        server_module._devforum_cache.clear()

        # Create cache with expired entry
        old_time = datetime.now(timezone.utc) - timedelta(hours=2)  # Older than TTL
        data = {
            "version": 1,
            "saved_at": datetime.now(timezone.utc).isoformat(),
            "entries": {
                "expired query": {
                    "query": "expired query",
                    "results": [{"id": 1}],
                    "timestamp": old_time.isoformat(),
                    "ttl_seconds": 3600,  # 1 hour
                }
            },
        }
        test_cache_path.write_text(json.dumps(data))

        # Load
        _load_devforum_cache()

        # Expired entry should not be loaded
        assert "expired query" not in server_module._devforum_cache

    def test_load_valid_entries(self, temp_cache_dir, monkeypatch):
        """Test valid entries are loaded."""
        import src.server as server_module

        test_cache_path = temp_cache_dir / "devforum_cache.json"
        monkeypatch.setattr(server_module, "_get_devforum_cache_path", lambda: test_cache_path)
        monkeypatch.setattr(server_module, "_devforum_cache_loaded", False)
        server_module._devforum_cache.clear()

        # Create cache with valid entry
        recent_time = datetime.now(timezone.utc) - timedelta(minutes=30)  # Within TTL
        data = {
            "version": 1,
            "saved_at": datetime.now(timezone.utc).isoformat(),
            "entries": {
                "valid query": {
                    "query": "valid query",
                    "results": [{"id": 123, "title": "Test"}],
                    "timestamp": recent_time.isoformat(),
                    "ttl_seconds": 3600,
                }
            },
        }
        test_cache_path.write_text(json.dumps(data))

        # Load
        _load_devforum_cache()

        # Valid entry should be loaded
        assert "valid query" in server_module._devforum_cache
        _, results = server_module._devforum_cache["valid query"]
        assert len(results) == 1
        assert results[0]["id"] == 123

    def test_version_mismatch_clears_cache(self, temp_cache_dir, monkeypatch):
        """Test version mismatch causes cache to be cleared."""
        import src.server as server_module

        test_cache_path = temp_cache_dir / "devforum_cache.json"
        monkeypatch.setattr(server_module, "_get_devforum_cache_path", lambda: test_cache_path)
        monkeypatch.setattr(server_module, "_devforum_cache_loaded", False)
        server_module._devforum_cache.clear()

        # Create cache with wrong version
        data = {
            "version": 999,  # Wrong version
            "entries": {"query": {"results": []}},
        }
        test_cache_path.write_text(json.dumps(data))

        # Load
        _load_devforum_cache()

        # Cache should be empty (version mismatch)
        assert len(server_module._devforum_cache) == 0
